local M = {}

local function create_objectlayer(layer, factories)
	local px_h = M.TILE_PX_HEIGHT * M.MAP_TILE_HEIGHT
	for i,v in ipairs(layer.objects) do
		local gpos = go.get_position()
		local pos = vmath.vector3( v.x + M.MAP_TILE_WIDTH, px_h - v.y + M.MAP_TILE_HEIGHT, 0 ) + gpos
		pos.z = 1
		factory.create(factories..v.name, pos)
	end
end

local function create_tilelayer(index, layer)
	local height = #layer.data / M.MAP_TILE_WIDTH
	for ii,vv in ipairs(layer.data) do	
		local pos_x = (ii-1) % M.MAP_TILE_WIDTH + 1
		local pos_y = height - (math.ceil(ii / M.MAP_TILE_WIDTH) - 1 )
		tilemap.set_tile(M.MAP_FILE, "layer" .. tostring(index), pos_x, pos_y, vv)
	end
end


local function load_map_data(def)
    local file2 = sys.load_resource(def)
    local def = json.decode(file2)
	return def
end

function M.create_level(def, container, factories)

	def = load_map_data(def)
	
	M.MAP_FILE = container
	M.MAP_TILE_WIDTH = def.width
	M.MAP_TILE_HEIGHT = def.height
	M.TILE_PX_WIDTH = def.tilewidth
	M.TILE_PX_HEIGHT = def.tileheight
	M.MAP_PX_WIDTH = M.TILE_PX_WIDTH * M.MAP_TILE_WIDTH
	M.MAP_PX_HEIGHT = M.TILE_PX_HEIGHT * M.MAP_TILE_HEIGHT
	

	local height = #def.layers[1].data / M.MAP_TILE_WIDTH
	for i,v in ipairs(def.layers) do		
		if v.type == "tilelayer" then	
			-- change tiles
			create_tilelayer(i,v)
		elseif v.type == "objectgroup" then
			-- add objects
			create_objectlayer(v, factories)
		end
	end

	
end

return M